import { Suspense, lazy, useEffect } from "react";
import { Toaster } from "@/components/ui/toaster";
import { Toaster as Sonner } from "@/components/ui/sonner";
import { TooltipProvider } from "@/components/ui/tooltip";
import { QueryClient, QueryClientProvider } from "@tanstack/react-query";
import { CookieProvider } from "@/contexts/CookieContext";
import CookieConsent from "@/components/CookieConsent";
import { BrowserRouter, Routes, Route, useLocation } from "react-router-dom";
import { HelmetProvider } from "react-helmet-async";

// Performance utilities
import { usePerformance } from "@/hooks/usePerformance";
import { initializePerformanceOptimizations } from "@/utils/performanceUtils";
import { initializeRoutePrefetching } from "@/hooks/useRoutePrefetch";

// Eagerly load only the Index (home) page and NotFound
import Index from "./pages/Index";
import NotFound from "./pages/NotFound";

// Lazy load all other routes
const Pricing = lazy(() => import("./pages/Pricing"));
const Features = lazy(() => import("./pages/Features"));
const Blogs = lazy(() => import("./pages/Blogs"));
const BlogPost = lazy(() => import("./pages/EnhancedBlogPost"));
const Admin = lazy(() => import("./pages/Admin"));
const Statistics = lazy(() => import("./pages/Statistics"));
const TermsAndConditions = lazy(() => import("./pages/TermsAndConditions"));
const PrivacyPolicy = lazy(() => import("./pages/PrivacyPolicy"));
const CookiePolicy = lazy(() => import("./pages/CookiePolicy"));
const ContactUs = lazy(() => import("./pages/ContactUs"));

// Simple fallback for lazy-loaded routes
const PageLoadingFallback = () => (
  <div className="min-h-screen flex items-center justify-center bg-gray-50">
    <div className="w-16 h-16 relative">
      <div className="animate-ping absolute h-full w-full rounded-full bg-orange-400 opacity-75"></div>
      <div className="relative rounded-full h-8 w-8 bg-orange-500 mx-auto my-4"></div>
    </div>
  </div>
);

// Configure React Query for performance
const queryClient = new QueryClient({
  defaultOptions: {
    queries: {
      refetchOnWindowFocus: false,
      staleTime: 5 * 60 * 1000, // 5 minutes
      gcTime: 10 * 60 * 1000, // 10 minutes (formerly cacheTime)
      retry: 1
    }
  }
});

// ScrollRestoration component to handle scroll position on navigation
const ScrollToTop = () => {
  const { pathname } = useLocation();
  
  useEffect(() => {
    window.scrollTo(0, 0);
    // Mark navigation performance
    performance.mark(`navigation:${pathname}`);
  }, [pathname]);
  
  return null;
};

// Main App Component
const App = () => {
  // Initialize performance optimizations
  useEffect(() => {
    // Initialize all performance optimizations
    if (typeof initializePerformanceOptimizations === 'function') {
      initializePerformanceOptimizations();
    }
    
    // Mark app mounted time
    performance.mark('app-mounted');
    performance.measure('app-mount-time', 'html-loaded', 'app-mounted');
  }, []);
  
  return (
    <QueryClientProvider client={queryClient}>
      <HelmetProvider>
        <CookieProvider>
          <TooltipProvider>
            <Toaster />
            <Sonner />
            <BrowserRouter>
              <ScrollToTop />
              <Routes>
                {/* Home route - eagerly loaded */}
                <Route
                  path="/"
                  element={<Index />}
                />

                {/* Lazily loaded routes with suspense */}
                <Route
                  path="/pricing"
                  element={
                    <Suspense fallback={<PageLoadingFallback />}>
                      <Pricing />
                    </Suspense>
                  }
                />

                <Route
                  path="/features"
                  element={
                    <Suspense fallback={<PageLoadingFallback />}>
                      <Features />
                    </Suspense>
                  }
                />

                <Route
                  path="/blogs"
                  element={
                    <Suspense fallback={<PageLoadingFallback />}>
                      <Blogs />
                    </Suspense>
                  }
                />

                <Route
                  path="/blog/:id"
                  element={
                    <Suspense fallback={<PageLoadingFallback />}>
                      <BlogPost />
                    </Suspense>
                  }
                />

                <Route
                  path="/terms-and-conditions"
                  element={
                    <Suspense fallback={<PageLoadingFallback />}>
                      <TermsAndConditions />
                    </Suspense>
                  }
                />

                <Route
                  path="/privacy-policy"
                  element={
                    <Suspense fallback={<PageLoadingFallback />}>
                      <PrivacyPolicy />
                    </Suspense>
                  }
                />
                
                <Route
                  path="/cookie-policy"
                  element={
                    <Suspense fallback={<PageLoadingFallback />}>
                      <CookiePolicy />
                    </Suspense>
                  }
                />

                <Route
                  path="/contact-us"
                  element={
                    <Suspense fallback={<PageLoadingFallback />}>
                      <ContactUs />
                    </Suspense>
                  }
                />

                <Route
                  path="/admin"
                  element={
                    <Suspense fallback={<PageLoadingFallback />}>
                      <Admin />
                    </Suspense>
                  }
                />

                <Route
                  path="/statistics"
                  element={
                    <Suspense fallback={<PageLoadingFallback />}>
                      <Statistics />
                    </Suspense>
                  }
                />

                {/* NotFound route - eagerly loaded */}
                <Route
                  path="*"
                  element={<NotFound />}
                />
              </Routes>
              <CookieConsent />
            </BrowserRouter>
          </TooltipProvider>
        </CookieProvider>
      </HelmetProvider>
    </QueryClientProvider>
  );
};

export default App;
